local name, addon = ...
local parentName = addon['parentName'] or "BrokerPack"
local childName = "zz_GarrisonToggle"
local realm = GetRealmName()
local player = UnitName('player').."-"..realm
local temp = {}
local defaults = {
	['garrisonText'] = 'GS',
	['garrisonHideCombat'] = false,
	['hideStandard'] = true,
}

local options = {
}

local db
-- local GBM = LE_FOLLOWER_TYPE_GARRISON_6_0
local GBM = LE_FOLLOWER_TYPE_GARRISON_7_0
local function OnText(message)
	addon['OnText'](childName, message)
end
local function switchDisplay(value)
	if(value) then
		AlertFrame:UnregisterEvent("GARRISON_MISSION_FINISHED")
		AlertFrame:UnregisterEvent("GARRISON_BUILDING_ACTIVATABLE")
	else
		AlertFrame:RegisterEvent("GARRISON_MISSION_FINISHED")
		AlertFrame:RegisterEvent("GARRISON_BUILDING_ACTIVATABLE")	
	end
end
local function getPref(pref)
	return db[pref[#pref]]
end

local function setPref(pref,value)
	if(pref[#pref] == "garrisonText" and value=="") then
		value = nil
	elseif(pref[#pref] == "garrisonHideCombat") then
		switchDisplay(value)
	end
	db[pref[#pref]] = value
	OnText(db['garrisonText'] or GARRISON_LOCATION_TOOLTIP)
end
local function OnButton(self,button)
	if(button=="LeftButton") then
		if(IsShiftKeyDown()) then
			GBM = GBM == LE_FOLLOWER_TYPE_GARRISON_6_0 and LE_FOLLOWER_TYPE_GARRISON_7_0 or LE_FOLLOWER_TYPE_GARRISON_6_0
			db['GBM'] = GBM
		else
			GarrisonLandingPageMinimapButton:Click("LeftButton")
		end
--		ToggleFrame(GarrisonLandingPage)
	elseif(button=="RightButton") then
		if(InterfaceOptionsFrame:IsVisible() and not InCombatLockdown()) then
			InterfaceOptionsFrame:Hide()
		else
			InterfaceOptionsFrame_OpenToCategory(childName)
		end
	end
end

local function IsInProgress(mid)
	for k,v in pairs(C_Garrison.GetInProgressMissions(temp, GBM) or {}) do
		if(tonumber(v['missionID'])==tonumber(mid)) then
			return true
		end
	end
	return false
end

local function GetMission(missionID)
	wipe(temp)
--	C_Garrison.GetInProgressMissions(temp, GBM)
	for k,v in pairs(C_Garrison.GetInProgressMissions(temp, GBM) or {}) do
		if(missionID==v['missionID']) then
			return v
		end
	end
end
local function AddDB(missionID)
	local missionInfo = GetMission(missionID)
	if(missionInfo) then
		local mt = {C_Garrison.GetMissionTimes(missionID)}
		db['missions'][player][GBM][missionID] = time() + mt[5]
	end
end
local function CheckDB(missionID)
	local missionInfo = GetMission(missionID)
	if(missionInfo) then
		local old = db['missions'][player][GBM][missionID]
		local mt = {C_Garrison.GetMissionTimes(missionID)}
		db['missions'][player][GBM][missionID] = db['missions'][player][GBM][missionID] or (time() + mt[5])
		if(db['missions'][player][GBM][missionID] ~= old) then
			print("RESET",missionInfo['name'])
		end
	end
end
local function prune()
	if(C_Garrison.GetInProgressMissions) then
		local ng = C_Garrison.GetInProgressMissions(temp, GBM)
		if((n and #n or 0)<1) then return end
		local ti = time()
		for k,v in pairs(db['missions'][player][GBM]) do
			if(not IsInProgress(k) and db['missions'][player][GBM][k] < ti) then
				db['missions'][player][GBM][k] = nil
			end
		end
	end
end

local function OnTooltipShow(GameTooltip)
	GameTooltip:AddLine(childName)
	local num = C_Garrison.GetNumFollowers(GBM)
	wipe(temp)
	C_Garrison.GetInProgressMissions(temp, GBM)

	sort(temp,function(a,b)
		return (db['missions'][player][GBM][a['missionID']] or 0) < (db['missions'][player][GBM][b['missionID']] or 0)
	end)
	local ti = time()
	for k,v in pairs(temp) do
		CheckDB(tonumber(v['missionID']))
		GameTooltip:AddDoubleLine(v['name'],db['missions'][player][GBM][tonumber(v['missionID'])]>ti and v['timeLeft'] or addon['colorize']("Done","00ff00"))
		num = num - v['numFollowers']
	end
	for k, v in pairs(db['missions']) do
		if(k ~= player and v[GBM]) then
			for a,b in pairs(v[GBM]) do
				if(b<ti) then
					GameTooltip:AddDoubleLine(k, addon['colorize']("Mission done","00ff00"))
					break
				end
			end
		end
	end
	if(IsShiftKeyDown()) then
		for k, v in pairs(db['missions']) do
			if(v[GBM]) then
				local found = 0
				for a,b in pairs(v[GBM]) do
					if(b>=ti) then
						local r = b - ti
						if(r<found or found==0) then
							found = r
						end
					end
				end
				if(found>0) then
					GameTooltip:AddDoubleLine(k, addon['colorize'](format("Next %.1f Min",found / 60 ),"ffff00"))
				end
			end
		end
	end
	local nr = C_Garrison.GetInProgressMissions(temp, GBM)
	local nr2 = C_Garrison.GetAvailableMissions(temp, GBM)
	GameTooltip:AddDoubleLine("Follower:",num.."/"..C_Garrison.GetNumFollowers(GBM))
	if(nr and nr2) then
		GameTooltip:AddDoubleLine("Missions:",#nr.."/"..#nr2)
	end
	GameTooltip:AddLine(addon['colorize']("SHIFT LEFTCLICK toggles Garrison","00ff00"))
	GameTooltip:AddLine(addon['colorize']("Press SHIFT for more info","00ff00"))
end
local function event(self,event,missionID,...)
	if(event == "GARRISON_MISSION_COMPLETE_RESPONSE") then
		local canComplete, succeeded = ...
		if(canComplete) then
--			db['missions'][player][GBM][missionID] = nil
		end
	elseif(event == "GARRISON_MISSION_STARTED") then
		AddDB(missionID)
	end
	prune()
end
local function init()
	db = addon['db']['profile'][childName]
	db['missions'] = db['missions'] or {}
	db['missions'][player] = db['missions'][player] or {
		[LE_FOLLOWER_TYPE_GARRISON_6_0] = {},
		[LE_FOLLOWER_TYPE_GARRISON_7_0] = {}
	}
	db['missions'][player][LE_FOLLOWER_TYPE_GARRISON_6_0] = db['missions'][player][LE_FOLLOWER_TYPE_GARRISON_6_0] or {}
	db['missions'][player][LE_FOLLOWER_TYPE_GARRISON_7_0] = db['missions'][player][LE_FOLLOWER_TYPE_GARRISON_7_0] or {}
	options = addon['InitConfig'](addon, childName, true, {
		['type'] = "launcher",
		['OnClick'] = OnButton,
		['text'] = db['garrisonText'] or GARRISON_LOCATION_TOOLTIP,
		['OnTooltipShow'] = OnTooltipShow,
		['icon'] = "Interface\\Icons\\Achievement_Arena_2v2_6",
	}, getPref, setPref)
	addon['AddConfigEntry'](addon,childName,'toggle','garrisonHideCombat',"Hide Alert","Hide mission complete alert",1,nil,nil,nil,nil)
	addon['AddConfigEntry'](addon,childName,'toggle','hideStandard',"Hide Blizzard","Hide Blizzards default garrison button",1,nil,nil,nil,nil)
	addon['AddConfigEntry'](addon,childName,'input','garrisonText',"Display Text","",1,nil,nil,nil,nil)
	addon['RegisterFunc']({"GARRISON_MISSION_STARTED","GARRISON_MISSION_FINISHED","GARRISON_MISSION_COMPLETE_RESPONSE","PLAYER_LOGOUT"},"OnEvent", event)
	if(not IsAddOnLoaded("Blizzard_GarrisonUI")) then
		UIParentLoadAddOn("Blizzard_GarrisonUI")
	end
	if(db['hideStandard']) then
		GarrisonLandingPageMinimapButton:UnregisterAllEvents()
		GarrisonLandingPageMinimapButton:Hide()
	end
	db['GBM'] = db['GBM'] or LE_FOLLOWER_TYPE_GARRISON_7_0
	GBM = db['GBM']
	switchDisplay(db['garrisonHideCombat'])
end
addon['startup'](addon, name, childName, init, true, defaults)